#include <regex>

//Defines the RX and TX pins for the cellular modem
#define RXD2 16
#define TXD2 17

#define relay1 23  //lamp light
#define relay2 19  //LED strip

String APN_ADDRESS = "internet";           //Enter APN of network You use
String AUTHORIZED_NUMBER = "+4591856841";  //Enter Your phone number with Your area prefix
String SIM_CARD_PIN_NUMBER = "";
String check_data = "";
String download_data = "";

const int MODEM_PWRKEY = 27;

//Function to initialize the modem
void configurationModem() {
  Serial2.println("ATV1");  //Set verbose mode on
  //Serial2.println("ATE1");           //Enable echo mode
  Serial2.println("AT+CMEE=2");      //Enable extended error reporting
  Serial2.println("AT+IPR=115200");  //Set baud rate to 115200
  Serial2.println("ATI");            // Get modem info
  if (SIM_CARD_PIN_NUMBER != "") {   //If SIM card requires a PIN, enter it
    Serial2.println((String) "AT+CPIN=" + SIM_CARD_PIN_NUMBER);
  }
  Serial2.println("AT+GSN");    //Get IMEI number
  Serial2.println("AT+CIMI");   //Get IMSI number
  Serial2.println("AT+QCCID");  //Get ICCID number
  Serial2.println((String) "AT+QICSGP=1,\"" + APN_ADDRESS + "\"");
  delay(100);
  download_data = (Serial2.readString());
  Serial.println();
  Serial.println(download_data);
  Serial.println();
  Serial.println("MODEM HAS BEEN CONFIGURED");
  download_data = "";
}

//Function to check the cellular network status
void networkCheck() {
  download_data = "";  //Initialize variable to hold network check data
  Serial.println("Network test");
  int connecting_count = 0;
  while (download_data == "") {
    Serial2.println("AT+CREG?");  //Send command to get network registration status
    download_data = Serial2.readString();
    if (download_data.indexOf("+CREG: 0,1") != -1)  //Check if device is registered on home network
    {
      Serial.println("Network registered - home");
      Serial.println(download_data);
      delay(50);
      break;
    } else if (download_data.indexOf("+CREG: 0,5") != -1)  //Check if device is registered on roaming network
    {
      Serial.println("Network registered - roaming");
      Serial.println(download_data);
      delay(50);
      break;
    } else if (download_data.indexOf("+CME ERROR: SIM failure") != -1)  //Check if there is a SIM failure error
    {
      Serial.println("SIM card error");
      Serial.println(download_data);
      download_data = Serial2.readString();
      download_data = "";
      delay(50);
    } else if (download_data.indexOf("+CME ERROR: 13") != -1)  //Check if there is a SIM failure error in CME numeric form
    {
      Serial.println("SIM card error");
      Serial.println(download_data);
      download_data = Serial2.readString();
      download_data = "";
      delay(50);
    } else {
      Serial.println("Connecting to network");
      Serial.println(download_data);
      download_data = "";
      delay(1000);
      connecting_count++;
    }
    if (connecting_count > 75) {
      setupModem();
      configurationModem();
      Serial.println("Some network connectivity issues... trying to connect again");
      delay(5000);
      connecting_count = 0;
    }
  }
}


//Function to setup a modem
void setupModem() {

  //Read any available data from the modem's serial interface
  download_data = Serial2.readString();

  //Check if the modem is already on or not
  Serial2.println("AT");  //Send command to check if modem is on
  download_data = Serial2.readString();
  if (download_data.indexOf("OK") > -1) {
    // Modem is already on
    Serial.println();
    Serial.println(download_data);
    Serial.println();
    Serial.println("Set modem startup:");
    Serial.println("Modem is aleady on");
    Serial.println();
    download_data = "";
  } else {
    //Modem was off, turn it on
    Serial.println();
    Serial.println(download_data);
    Serial.println("Set modem startup::");
    Serial.println("The modem has been switched on");
    Serial.println();
    digitalWrite(MODEM_PWRKEY, HIGH);  //Turning ON modem by set power on PWR_KEY pin through 1000 ms
    delay(1000);
    digitalWrite(MODEM_PWRKEY, LOW);
    Serial2.println("ATE1");  //Set echo text mode on modem
    download_data = "";
    download_data = Serial2.readString();
    int initial_counter = 0;
    delay(1000);
    while (download_data.indexOf("Call Ready") == -1) {
      Serial.println("Waiting for modem init");
      delay(500);
      download_data = Serial2.readString();
      Serial.println(download_data);
      initial_counter++;
      if (initial_counter > 10)  //Trying to turn up modem again
      {
        digitalWrite(MODEM_PWRKEY, HIGH);
        delay(1000);
        digitalWrite(MODEM_PWRKEY, LOW);
        initial_counter = 0;
      }
    }
  }
}



void receiveSMS() {
  Serial2.println("AT+CMGF=1");  // Set message format to text mode
  delay(100);
  Serial2.println("AT+CSCS=\"GSM\"");  // Set the character set to GSM
  delay(50);
  Serial2.println("AT+CNMI=2,1");  // Set module to notify on new SMS messages
  delay(100);
  Serial.println("Waiting for incoming SMS message...");  // Set module to notify on new SMS messages


  if (Serial2.available()) {
    // Read the response from the GSM module
    String response = Serial2.readString();
    // Check if the response contains the "+CMTI:" indicator, which means a new message has arrived
    if (response.indexOf("+CMTI:") != -1) {
      // Retrieve the message content using the message index
      Serial2.println("AT+CMGR=1");
      delay(100);
      String messageContent = Serial2.readString();
      // Print the received SMS message to the Serial Monitor
      Serial.println("Received SMS message:");
      Serial.println(messageContent);
      Serial2.println("AT+CMGD=1,4");
      // Extract the sender's phone number from the response
      int firstComma = messageContent.indexOf(',') + 1;                        // Find the position of the first comma
      int secondComma = messageContent.indexOf(',', firstComma + 1);           // Find the position of the second comma
      String phoneNumber = messageContent.substring(firstComma, secondComma);  // Extract the phone number                               
      phoneNumber.remove(0, 1);                                                // Remove the first double quote
      phoneNumber.remove(phoneNumber.length() - 1);                            // Remove the last double quote
      Serial.println(phoneNumber);                                             // Print the extracted phone number


      if (phoneNumber == AUTHORIZED_NUMBER) {
        // Use Regular Expressions to extract the user's intent
        std::regex turnOnRegex("TURN ON LAMP");
        std::regex turnOffRegex("TURN OFF LAMP");
        std::smatch match;

        std::regex turnOnRegex2("TURN ON LED");
        std::regex turnOffRegex2("TURN OFF LED");
        std::smatch match2;


        // Convert messageContent to a std::string before passing it to std::regex_search
        std::string messageContentStr(messageContent.c_str());

        // Check if the message contains the "TURN ON LAMP" keyword
        if (std::regex_search(messageContentStr, match, turnOnRegex)) {
          digitalWrite(relay1, HIGH);  // Turn on the relay
          Serial.println("Lamp turned on");
        }
        // Check if the message contains the "TURN OFF LAMP" keyword
        else if (std::regex_search(messageContentStr, match, turnOffRegex)) {
          digitalWrite(relay1, LOW);  // Turn off the relay
          Serial.println("Lamp turned off");
        }
        // Check if the message contains the "TURN OFF LED" keyword
        else if (std::regex_search(messageContentStr, match2, turnOnRegex2)) {
          digitalWrite(relay2, HIGH);  // Turn off the relay
          Serial.println("LED strip turned on");
        }
        // Check if the message contains the "TURN OFF LED" keyword
        else if (std::regex_search(messageContentStr, match2, turnOffRegex2)) {
          digitalWrite(relay2, LOW);  // Turn off the relay
          Serial.println("LED strip turned off");
        }
        // If the message does not contain any recognized keywords, print an error message
        else {
          Serial.println("Unknown command");
        }
      } else {
        Serial.println("NOT AUTHORIZED NUMBER TRIED TO ENTER!");
      }
    }
  }
  delay(100);  // Wait for 100ms before checking again
}

//Function to send an SMS message
void sendSMS(String phone_number, String message) {
  Serial2.println("AT+CMGF=1");                                  //Setup type of text message
  Serial2.println("AT+CSCS=\"GSM\"");                            //Select character set
  Serial2.println((String) "AT+CMGS=\"" + phone_number + "\"");  //Send SMS in text mode
  Serial2.println(message);
  Serial2.write(26);
  Serial2.println();
  //Wait for a response from the modem
  delay(2000);  //Adjust the delay time as needed
  String response = (Serial2.readString());
  Serial.println(response);
  if (response.indexOf("+CMGS:") != -1) {
    Serial.println("SMS message sent successfully!");
    Serial.println("Modem sent SMS message to:");
    Serial.println(AUTHORIZED_NUMBER);
    Serial.println("SMS content:");
    Serial.println(message);
    Serial.println();
  }
}


void setup() {
  Serial.begin(115200);
  Serial.setTimeout(100);
  Serial2.begin(115200, SERIAL_8N1, RXD2, TXD2);
  delay(500);
  pinMode(MODEM_PWRKEY, OUTPUT);
  Serial2.setTimeout(100);
  Serial.println("----------- MICROMIS POWER SHIELD start! -----------");
  pinMode(relay1, OUTPUT);
  pinMode(relay2, OUTPUT);
  setupModem();
  configurationModem();
  networkCheck();
}

void loop() {
  receiveSMS();
}
